package sun.rmi.runtime;

import java.security.AccessController;
import sun.security.util.SecurityConstants;
import java.security.PrivilegedAction;

public final class NewThreadAction implements PrivilegedAction<Thread>
{
    static final ThreadGroup systemThreadGroup;
    static final ThreadGroup userThreadGroup;
    private final ThreadGroup group;
    private final Runnable runnable;
    private final String name;
    private final boolean daemon;
    
    NewThreadAction(ThreadGroup group, Runnable runnable, String name, boolean daemon) {
        this.group = group;
        this.runnable = runnable;
        this.name = name;
        this.daemon = daemon;
    }
    
    public NewThreadAction(Runnable runnable, String s, boolean b) {
        this(NewThreadAction.systemThreadGroup, runnable, s, b);
    }
    
    public NewThreadAction(Runnable runnable, String s, boolean b, boolean b2) {
        this(b2 ? NewThreadAction.userThreadGroup : NewThreadAction.systemThreadGroup, runnable, s, b);
    }
    
    @Override
    public Thread run() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
        }
        Thread thread = new Thread(this.group, this.runnable, "RMI " + this.name);
        thread.setContextClassLoader(ClassLoader.getSystemClassLoader());
        thread.setDaemon(this.daemon);
        return thread;
    }
    
    static {
        systemThreadGroup = (ThreadGroup)AccessController.doPrivileged(new PrivilegedAction<ThreadGroup>() {
            @Override
            public ThreadGroup run() {
                ThreadGroup threadGroup;
                ThreadGroup parent;
                for (threadGroup = Thread.currentThread().getThreadGroup(); (parent = threadGroup.getParent()) != null; threadGroup = parent) {}
                return threadGroup;
            }
        });
        userThreadGroup = (ThreadGroup)AccessController.doPrivileged(new PrivilegedAction<ThreadGroup>() {
            @Override
            public ThreadGroup run() {
                return new ThreadGroup(NewThreadAction.systemThreadGroup, "RMI Runtime");
            }
        });
    }
}
